# Django
from django.db.models.expressions import F

# Standard Library
from datetime import date

# MuckRock
from muckrock.daily_active_users.models import DailyActiveUser


class DailyActiveUserMiddleware:
    """Middleware to log each user once per day"""

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        """Ensure there is a single record for the active user for today's date"""

        if request.user.is_authenticated:
            organizations = list(
                request.user.organizations.values(
                    "id", "name", plan=F("entitlement__name")
                )
            )
            DailyActiveUser.objects.get_or_create(
                user=request.user,
                date=date.today(),
                defaults={"metadata": {"organizations": organizations}},
            )

        return self.get_response(request)



# Generated by Django 4.2 on 2024-11-05 13:57

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="DailyActiveUser",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(default=datetime.date.today)),
                ("metadata", models.JSONField(default=dict)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "unique_together": {("user", "date")},
            },
        ),
    ]



# Django
from django.db import models

# Standard Library
from datetime import date


class DailyActiveUser(models.Model):
    """A log of user activity per day"""

    user = models.ForeignKey(
        "auth.User",
        on_delete=models.PROTECT,
        related_name="+",
    )
    date = models.DateField(default=date.today)
    metadata = models.JSONField(default=dict)

    class Meta:
        unique_together = [("user", "date")]



# Django
from django.contrib import admin
from django.http.response import HttpResponse
from django.utils.safestring import mark_safe

# Standard Library
import csv
import json
from itertools import chain

# MuckRock
from muckrock.daily_active_users.models import DailyActiveUser


@admin.register(DailyActiveUser)
class DailyActiveUserAdmin(admin.ModelAdmin):
    """Daily Active User admin"""

    date_hierarchy = "date"
    list_display = ("user", "date")
    list_select_Related = ("user",)
    readonly_fields = ("user", "date", "formatted_metadata")
    fields = ("user", "date", "formatted_metadata")
    actions = ["export_as_csv"]

    def formatted_metadata(self, obj):
        json_data = json.dumps(obj.metadata, indent=4)
        return mark_safe(f"<pre>{json_data}</pre>")

    formatted_metadata.short_description = "Metadata"

    def export_as_csv(self, request, queryset):
        """Export daily active users"""

        meta = self.model._meta
        field_names = [
            "user",
            "date",
        ]

        response = HttpResponse(content_type="text/csv")
        response["Content-Disposition"] = f"attachment; filename={meta}.csv"
        writer = csv.writer(response)

        writer.writerow(field_names)
        for obj in queryset:
            writer.writerow(
                [getattr(obj, field) for field in field_names]
                + list(
                    chain(
                        *[[m["name"], m["plan"]] for m in obj.metadata["organizations"]]
                    )
                )
            )

        return response

    export_as_csv.short_description = "Export Selected"



# Third Party
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="daily_active_users",  # Replace with your own username
    version="0.1.0",
    author="Mitchell Kotler",
    author_email="mitch@muckrock.com",
    description="Django logging of daily active users",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/muckrock/daily-active-users/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "django",
    ],
    python_requires=">=3.6",
)
